# SPDX-FileCopyrightText: 2021 Dimitris Kardarakos <dimkard@posteo.net>
# SPDX-License-Identifier: GPL-3.0-or-later

from abc import ABC, abstractmethod


class AbstractConfigurator(ABC):
    """Abstract class that represents the interface that Glossaico configuration
    should implement
    """

    @staticmethod
    @abstractmethod
    def dataDirectory():
        """Returns the directory to which user data should be written"""

    @staticmethod
    @abstractmethod
    def cacheDirectory():
        """Returns the directory to which non-essential user data
        should be written
        """

    @staticmethod
    @abstractmethod
    def configDirectory():
        """Returns the directory to which configuration files
        should be written
        """

    @staticmethod
    @abstractmethod
    def applicationDirectory():
        """Returns the base directory of the application executable"""

    @abstractmethod
    def userSkillStats(self, uid, course_name, skill_id):
        """Returns the statistics object that represents the progress of a user
        for a specific course skill
        """

    @abstractmethod
    def updateUserSkillStats(self, uid, course_name, skill_id,
                             challenges_passed, total_skill_challenges):
        """Updates the statistics object that represents the progress of a user
        for a specific course skill
        """

    @abstractmethod
    def activeUserSkillStats(self, skill_id):
        """Returns the statistics object that represents the progress of
        the active user for a specific course skill
        """

    @abstractmethod
    def updateActiveUserSkillStats(self, skill_id,
                                   challenges_passed, total_skill_challenges):
        """Updates the statistics object that represents the progress of
        the active user for a specific course skill
        """

    @property
    @abstractmethod
    def configVersion(self):
        """Returns the version of the configuration schema. It is useful when
        applications are updated and configuration data should be migrated.
        """

    @configVersion.setter
    @abstractmethod
    def configVersion(self, val):
        pass

    @property
    @abstractmethod
    def activeUserId(self):
        """Returns the user id of the active user"""

    @activeUserId.setter
    @abstractmethod
    def activeUserId(self, val):
        pass

    @property
    @abstractmethod
    def activeCourseName(self):
        """Returns the name of the course currently in use"""

    @activeCourseName.setter
    @abstractmethod
    def activeCourseName(self, val):
        pass

    @property
    @abstractmethod
    def silentMode(self):
        """Returns True if no sound should be reproduced by the application"""

    @silentMode.setter
    @abstractmethod
    def silentMode(self, val):
        pass

    @property
    def imagesReady(self):
        """Returns True if the pictures needed by the application courses
        have been downloaded
        """

    @imagesReady.setter
    @abstractmethod
    def imagesReady(self, val):
        pass
