# SPDX-FileCopyrightText: 2021 Dimitris Kardarakos <dimkard@posteo.net>
# SPDX-License-Identifier: GPL-3.0-or-later

from PySide2.QtCore import QObject, Signal, Property
from glossaico.yaml_configurator import YamlConfigurator


class QConfigSingletonAdapter(QObject):
    """Class that manages the configuration of the application
    and exposes qml properties
    """

    updated = Signal()
    _instance = None

    def __init__(self):
        if QConfigSingletonAdapter._instance:
            raise Exception("QConfigSingletonAdapter is a singleton class")

        QConfigSingletonAdapter._instance = self
        QObject.__init__(self)
        self._cfg = YamlConfigurator()

    @staticmethod
    def getInstance():
        """Returns the single object that can be instantiated"""
        if not QConfigSingletonAdapter._instance:
            QConfigSingletonAdapter()

        return QConfigSingletonAdapter._instance

    def readDataDirectory(self):
        """Returns the directory to which user data should be written"""
        return YamlConfigurator.dataDirectory()

    def readApplicationDirectory(self):
        """Returns the base directory of the application executable"""
        return YamlConfigurator.applicationDirectory()

    def readCacheDirectory(self):
        """Returns the directory to which non-essential user data
        should be written
        """
        return YamlConfigurator.cacheDirectory()

    def readActiveUserId(self):
        """Returns the user id of the active user"""
        return self._cfg.activeUserId

    def setActiveUserId(self, val):
        """Sets the user id of the active user"""
        self._cfg.activeUserId = val
        self.activeUserIdChanged.emit()

    @Signal
    def activeUserIdChanged(self):
        """Emitted when the user id of the active user is set"""

    def readActiveCourseName(self):
        """Returns the name of the course currently in use"""
        return self._cfg.activeCourseName

    def setActiveCourseName(self, val):
        """Sets the name of the course currently in use"""
        self._cfg.activeCourseName = val
        self.activeCourseNameChanged.emit()

    @Signal
    def activeCourseNameChanged(self):
        """Emitted when the course currently in use changes"""

    def readSilentMode(self):
        """Returns True if no sound should be reproduced by the application"""
        return self._cfg.silentMode

    def setSilentMode(self, val):
        """Controls the configuration option that should be set to True
        if no sound should be reproduced by the application"""
        self._cfg.silentMode = val
        self.silentModeChanged.emit()

    @Signal
    def silentModeChanged(self):
        """Emitted when the silent mode property is set"""

    def readImagesReady(self):
        """Returns True if the pictures needed by the application courses
        have been downloaded
        """
        return self._cfg.imagesReady

    def setImagesReady(self, val):
        """Controls the configuration option that should be set to True
        if the pictures needed by the application courses have been downloaded
        """
        self._cfg.imagesReady = val
        self.imagesReadyChanged.emit()

    @Signal
    def imagesReadyChanged(self):
        """Emitted when the image ready property is set"""

    def userSkillStats(self, user_id, course_name, skill_id):
        """Returns the statistics object that represents the progress of a user
        for a specific course skill
        """
        return self._cfg.userSkillStats(user_id, course_name, skill_id)

    def updateUserSkillStats(self, user_id, course_name, skill_id,
                             challenges_passed, total_skill_challenges):
        """Updates the statistics object that represents the progress of a user
        for a specific course skill
        """
        self._cfg.updateUserSkillStats(
            user_id, course_name, skill_id,
            challenges_passed, total_skill_challenges)
        self.updated.emit()

    def activeUserSkillStats(self, skill_id):
        """Returns the statistics object that represents the progress of the
        active user for a specific course skill
        """
        return self._cfg.activeUserSkillStats(skill_id)

    def updateActiveUserSkillStats(self, skill_id,
                                   challenges_passed, total_skill_challenges):
        """Updates the statistics object that represents the progress of
        the active user for a specific course skill
        """
        self._cfg.updateActiveUserSkillStats(
            skill_id, challenges_passed, total_skill_challenges)
        self.updated.emit()

    activeUserId = Property(str, readActiveUserId,
                            setActiveUserId, notify=activeUserIdChanged)
    activeCourseName = Property(
        str, readActiveCourseName, setActiveCourseName,
        notify=activeCourseNameChanged)
    dataDirectory = Property(str, readDataDirectory, constant=True)
    cacheDirectory = Property(str, readCacheDirectory, constant=True)
    applicationDirectory = Property(
        str, readApplicationDirectory, constant=True)
    silentMode = Property(bool, readSilentMode,
                          setSilentMode, notify=silentModeChanged)
    imagesReady = Property(bool, readImagesReady,
                           setImagesReady, notify=imagesReadyChanged)
