# SPDX-FileCopyrightText: 2021 Dimitris Kardarakos <dimkard@posteo.net>
# SPDX-License-Identifier: GPL-3.0-or-later

from pathlib import Path
import hashlib
import zipfile
import re
from threading import Thread
from concurrent.futures import ThreadPoolExecutor
from os import listdir
from os.path import isfile, join
from os.path import exists as file_exists
import requests
from PySide2.QtCore import QObject, Signal, Slot, Property
from glossaico.q_config_singleton_adapter import QConfigSingletonAdapter


class QDataController(QObject):
    """Class that manages the data (images, audios) needed"""
    ll_web_release = {
        "name": "librelingo-web-v1.29.2.zip",
        "url": "https://github.com/kantord/LibreLingo/archive/refs/tags/@librelingo/web-v1.29.2.zip",
        "sha256sum": "e2fdf7e214addb6de7ce673ea2883c13fb0ebb87c2e50798e023ccb12c61312e"
        }
    _instance = None
    imageFetchCompleted = Signal()

    def __init__(self):
        if QDataController._instance:
            raise Exception("QDataController is a singleton class")

        QDataController._instance = self
        QObject.__init__(self)

        self._cfg = QConfigSingletonAdapter.getInstance()
        self._active_download_name = ""

    @staticmethod
    def getInstance():
        """Returns the single object that can be instantiated"""
        if not QDataController._instance:
            QDataController()

        return QDataController._instance

    def fetchAudios(self, audios_needed, course_name):
        """Downloads the audio files needed by a language course"""
        llm_local_file = self._fetchReleaseArchive()

        audio_folder = Path(self._cfg.dataDirectory, course_name, 'voice')

        if not audio_folder.is_dir():
            print("Voice folder does not exist, create a new one")
            audio_folder.mkdir(parents=True)

        # List of the audios already fetched
        audios_in_filesystem = [f for f in listdir(audio_folder)
                                if isfile(join(audio_folder, f))]

        # Create list of audios to fetch
        audios_to_extract = ["{}.mp3".format(a[0]) for a in audios_needed
                             if "{}.mp3".format(a[0]) not in audios_in_filesystem]

        self.extractFilesToDir(llm_local_file, audio_folder,
                               file_list=audios_to_extract)

    def downloadFile(self, download_name, remote_url, file_path):
        """Downloads a file from a remote url and saves it as a local file"""
        self.setActiveDownloadName(download_name)
        req = requests.get(remote_url)

        if req.status_code != 200:
            print("Cannot download from", remote_url)
            raise requests.HTTPError("File cannot be found")
        with open(file_path, 'wb') as file:
            file.write(req.content)

    def fetchImages(self):
        """Downloads the image files needed by a language course"""
        llm_local_file = self._fetchReleaseArchive()

        dst = Path(self._cfg.dataDirectory, "images")
        if not dst.is_dir():
            dst.mkdir()

        self.extractFilesToDir(llm_local_file, dst,
                               file_pattern="^.*images/.*.jpg$")
        self._cfg.imagesReady = True
        self.imageFetchCompleted.emit()

    def _fetchReleaseArchive(self):
        llm_local_file = Path("{}/{}".format(Path(self._cfg.cacheDirectory),
                                             QDataController.ll_web_release["name"]))

        if (not file_exists(llm_local_file) or (self._digestSha256Sum(llm_local_file)
                                                != QDataController.ll_web_release["sha256sum"])):
            with ThreadPoolExecutor() as executor:
                executor.submit(self.downloadFile, QDataController.ll_web_release["name"],
                                QDataController.ll_web_release["url"], llm_local_file)

        if self._digestSha256Sum(llm_local_file) != QDataController.ll_web_release["sha256sum"]:
            raise Exception("The checksum of the downloaded file is invalid")

        return llm_local_file

    def extractFilesToDir(self, local_file, destination, file_pattern="", file_list=None):
        """Extracts files from a ZIP archive to the destination directory path
        provided.
        If only a file pattern is provided, the files that match the pattern are extracted.
        If only a file list is provided, the files that exists in the list are extracted.
        If both a pattern and a list are provided, the files that satisfy both criteria
        are extracted.
        """
        regex_pattern = None

        if file_pattern:
            regex_pattern = re.compile(file_pattern)

        with zipfile.ZipFile(local_file) as z:
            for member_name in z.namelist():
                member_path = Path(member_name)
                file_name = member_path.name
                if (((not regex_pattern) or regex_pattern.match(member_name))
                        and ((not file_list) or (file_name in file_list))):
                    dst_path = Path(destination, file_name)
                    with open(dst_path, 'wb') as dst_file:
                        dst_file.write(z.read(member_name))

    def _digestSha256Sum(self, file_path):
        check_sum = ""
        with open(file_path, "rb") as f:
            bytes_read = f.read()
            check_sum = hashlib.sha256(bytes_read).hexdigest()

        return check_sum

    def readActiveDownloadName(self):
        """Returns the name of the file being downloaded"""
        download_file = ""
        if self._active_download_name:
            download_file = self._active_download_name

        return download_file

    def setActiveDownloadName(self, val):
        """Sets the name of the file being downloaded"""
        self._active_download_name = val
        self.activeDownloadNameChanged.emit()

    @Signal
    def activeDownloadNameChanged(self):
        """Emitted when a new file is being downloaded"""

    @Slot()
    def initiliazeImageData(self):
        """Manages the download of the image files needed by a language course"""
        self._active_download_name = ""
        thread = Thread(target=self.fetchImages)
        thread.start()

    def audioExists(self, course_name, audio_name):
        """Returns True if the audio provided is found in the filesystem"""
        audio_file = Path(self._cfg.dataDirectory, course_name,
                          'voice', "{}.mp3".format(audio_name))

        return file_exists(audio_file)

    def imageExists(self, image_name, image_size):
        """Returns True if the image provided is found in the filesystem.
        Image sizes: normal, tiny, tinier"""
        size_pattern = ".jpg" if image_size == "normal" else "_{}.jpg".format(image_size)
        image_file = Path(self._cfg.dataDirectory, "images",
                          image_name.replace(".jpg", size_pattern))
        return file_exists(image_file)

    activeDownloadName = Property(
        str, readActiveDownloadName, setActiveDownloadName, notify=activeDownloadNameChanged)
