# SPDX-FileCopyrightText: 2021 Dimitris Kardarakos <dimkard@posteo.net>
# SPDX-License-Identifier: GPL-3.0-or-later

from pathlib import Path
from threading import Thread
import traceback
import sys
from librelingo_yaml_loader import load_course
from PySide2.QtCore import QObject, Signal, Slot, Property
from librelingo_audios import list_required_audios
from glossaico.q_data_controller import QDataController
from glossaico.q_config_singleton_adapter import QConfigSingletonAdapter


class QLanguageCourse(QObject):
    """Class that provides a language course"""

    loaded = Signal()

    def __init__(self):
        QObject.__init__(self)
        self._course = []
        self._course_name = ""
        self._cfg = QConfigSingletonAdapter.getInstance()
        self._data_folder_path = Path(self._cfg.readDataDirectory())
        self._q_data_controller = QDataController.getInstance()
        self._course_yaml_path = ""

        self.loaded.connect(self.targetLanguageNameChanged)
        self.loaded.connect(self.targetLanguageCodeChanged)
        self.loaded.connect(self.sourceLanguageChanged)
        self.loaded.connect(self.specialCharactersChanged)
        self.loaded.connect(self.modulesChanged)

    @Slot(str)
    def load(self, name):
        """Loads the data of the language course provided"""
        self._course_name = name
        self._course_yaml_path = ""
        self.nameChanged.emit()
        self._cfg.activeCourseName = self._course_name
        print("Selected course: {0}".format(self._course_name))
        thread = Thread(target=self._loadData)
        thread.start()

    def _loadData(self):
        if not self._course_name:
            return

        self._course_yaml_path = "{}/courses/yaml/{}/course".format(
            self._cfg.applicationDirectory, self._course_name)

        # Load the language course
        try:
            self._course = load_course(self._course_yaml_path)
        except Exception:
            print(traceback.format_exc())
            print("Course cannot be loaded, exiting application")
            sys.exit(1)

        config_audio_settings = self._course.settings.audio_settings if hasattr(
            self._course.settings, "audio_settings") else None

        # Fetch voice data
        if config_audio_settings and config_audio_settings.enabled:
            required_audios = list_required_audios(self._course)
            self._q_data_controller.fetchAudios(
                required_audios, self._course_name)

        self.loaded.emit()

    def readName(self):
        """Returns the name of the language course"""
        return self._course_name

    @Signal
    def nameChanged(self):
        """Emitted when the course name is set"""

    def readTargetLanguageName(self):
        """Returns the name of the target language of the course"""
        language = ""
        if self._course:
            language = self._course.target_language.name

        return language

    @Signal
    def targetLanguageNameChanged(self):
        """Emitted when the target language of the course is set"""

    def readTargetLanguageCode(self):
        """Returns the code of the target language of the course"""
        code = ""
        if self._course:
            code = self._course.target_language.code

        return code

    @Signal
    def targetLanguageCodeChanged(self):
        """Emitted when the code of the target language of the course is set"""

    def readSourceLanguage(self):
        """Returns the name of the source language of the course"""
        language = ""
        if self._course:
            language = self._course.source_language

        return language

    @Signal
    def sourceLanguageChanged(self):
        """Emitted when the source language of the course is set"""

    def readSpecialCharacters(self):
        """Returns the list of the special characters needed by the
        language course
        """
        special_chars = []
        if self._course:
            special_chars = self._course.special_characters

        return special_chars

    @Signal
    def specialCharactersChanged(self):
        """Emitted when the special characters of the course are set"""

    def readModules(self):
        """Returns the list of the modules included in the language course"""
        mods = []
        if self._course:
            mods = self._course.modules

        return mods

    @Signal
    def modulesChanged(self):
        """Emitted when the list of the modules of the course is set"""

    name = Property(str, readName, notify=nameChanged)
    targetLanguageName = Property(
        str, readTargetLanguageName, notify=targetLanguageNameChanged)
    targetLanguageCode = Property(
        str, readTargetLanguageCode, notify=targetLanguageCodeChanged)
    sourceLanguage = Property(list, readSourceLanguage,
                              notify=sourceLanguageChanged)
    specialCharacters = Property(
        list, readSpecialCharacters, notify=specialCharactersChanged)
    courseModules = Property(list, readModules, notify=modulesChanged)
