# SPDX-FileCopyrightText: 2021 Dimitris Kardarakos <dimkard@posteo.net>
# SPDX-License-Identifier: GPL-3.0-or-later

from PySide2.QtCore import QObject, Signal, Slot, Property


class QPracticeContext(QObject):
    """This class is the practice context used by the QML user interface.
    It can make use of various strategies that implement the
    AbstractPracticeStrategy interface.
    """

    sessionEarlyFinished = Signal()

    def __init__(self, practice_strategy):
        QObject.__init__(self)

        self._practice_strategy = practice_strategy
        self._practicing = False

    @Slot()
    def finalizeSession(self):
        """It is called when a learning session -consisting of a set of
        challenges- ends, e.g. because all challenges in the session have
        been answered or skipped, or the user opts to finish early.
        """
        self._practice_strategy.updateStats()
        self.completeSession.emit()

    @Slot()
    def completeChallenge(self):
        """It is called when a challenge has finished, after having been
        answered or skipped by the user, and the users is moving on to
        the next challenge
        """
        if self.loadChallenge():
            self.continueSession.emit()
        else:
            self.finalizeSession()

    @Signal
    def continueSession(self):
        """It is emitted when a new challenge has been loaded"""

    @Signal
    def completeSession(self):
        """It is emitted when a learning session -consisting of a set of
        challenges- has been finalized
        """

    def readCurrentChallengeQuestion(self):
        """Returns the question of the current challenge"""
        return self._practice_strategy.currentChallengeQuestion

    @Signal
    def currentChallengeQuestionChanged(self):
        """Emitted when the current challenge has changed"""

    @Slot(result=float)
    def practiceProgress(self):
        """Returns the percentage of the practice session that has been
        completed
        """
        return self._practice_strategy.practiceProgress

    def readChallengesAnswered(self):
        """Returns the amount of challenges already answered in the current
        practice session
        """
        return self._practice_strategy.challengesAnswered

    @Signal
    def challengesAnsweredChanged(self):
        """Emitted when the amount of challenges already answered in the
        current practice session has changed
        """

    @Slot(str, str, str, list, list, "QVariantMap", result=bool)
    def startPractice(self, course_name, skill_id, skill_name, skill_words,
                      skill_phrases, user_stats):
        """Executes the required initialization steps needed for a practice
        session for a specific skill to be started
        """
        self._practicing = self._practice_strategy.startPractice(
            course_name, skill_id, skill_name, skill_words,
            skill_phrases, user_stats)
        self.practicingChanged.emit()

        return self._practicing

    @Slot(result=bool)
    def endPractice(self):
        """It is called when the user exits practicing"""
        self._practicing = False
        self.practicingChanged.emit()

    @Slot(result=bool)
    def loadChallenge(self):
        """If there are remaining challenges in the practice session, it
        initializes the challenge to be offered to the user, e.g.
        currentChallengeQuestion should be set accordingly. Returns True
        if such a challenge exists.
        """
        load_result = self._practice_strategy.loadChallenge()
        self.currentChallengeQuestionChanged.emit()

        return load_result

    @Slot("QVariantMap", result=bool)
    def checkResult(self, answer_given):
        """Returns True if the answer given is the correct one for the current
        challenge
        """
        is_correct = self._practice_strategy.checkResult(answer_given)
        self.challengesAnsweredChanged.emit()

        return is_correct

    def readPracticing(self):
        """Returns True is the user is in a practice session"""
        return self._practicing

    @Signal
    def practicingChanged():
        """Emitted when the user enters or exits a practice session"""

    currentChallengeQuestion = Property(
        "QVariantMap", readCurrentChallengeQuestion, notify=currentChallengeQuestionChanged)
    challengesAnswered = Property(
        int, readChallengesAnswered, notify=challengesAnsweredChanged)
    practicing = Property(bool, readPracticing, notify=practicingChanged)
