# SPDX-FileCopyrightText: 2021 Dimitris Kardarakos <dimkard@posteo.net>
# SPDX-License-Identifier: GPL-3.0-or-later

from PySide2.QtCore import Signal, Slot, Property, QAbstractListModel, QModelIndex, Qt
from slugify import slugify
from glossaico.q_config_singleton_adapter import QConfigSingletonAdapter


class QSkillsModel(QAbstractListModel):
    """Class that provides a model of the skills of a course"""

    NameRole = Qt.UserRole + 1
    IdRole = Qt.UserRole + 2
    WordsRole = Qt.UserRole + 3
    PhrasesRole = Qt.UserRole + 4
    ImageSetRole = Qt.UserRole + 5
    DictionaryRole = Qt.UserRole + 6
    IntroductionRole = Qt.UserRole + 7
    SampleWordsStrRole = Qt.UserRole + 8
    SamplePhrasesStrRole = Qt.UserRole + 9
    SampleImageRole = Qt.UserRole + 10
    ActiveUserProgress = Qt.UserRole + 11
    ActiveUserSkillStats = Qt.UserRole + 12

    def __init__(self, parent=None):
        super().__init__(parent)
        self._skills = ""
        self._cfg = QConfigSingletonAdapter.getInstance()
        self._cfg.updated.connect(self.updateModel)

    def roleNames(self):
        """Returns the model's role names"""
        return {
            QSkillsModel.NameRole: b'name',
            QSkillsModel.IdRole: b'skill_id',
            QSkillsModel.WordsRole: b'words',
            QSkillsModel.PhrasesRole: b'phrases',
            QSkillsModel.ImageSetRole: b'skill_images',
            QSkillsModel.DictionaryRole: b'skills_dictionary',
            QSkillsModel.IntroductionRole: b'introduction',
            QSkillsModel.SampleWordsStrRole: b'sample_words',
            QSkillsModel.SamplePhrasesStrRole: b'sample_phrases',
            QSkillsModel.SampleImageRole: b'sample_image',
            QSkillsModel.ActiveUserProgress: b'active_user_progress',
            QSkillsModel.ActiveUserSkillStats: b'active_user_skill_stats',
        }

    def rowCount(self, parent=QModelIndex()):
        """Returns the number of rows under the given parent"""
        if parent.isValid():
            return 0

        return len(self._skills)

    def data(self, index, role):
        """Returns the data stored under the given role for the item referred to by the index"""
        if not index.isValid():
            return None

        row = self._skills[index.row()]
        skill_id = "{}_{}".format(row.id, slugify(row.name))
        stats = self._cfg.activeUserSkillStats(skill_id)
        result = "Invalid Role"

        if role == Qt.DisplayRole:
            result = row.name
        elif role == QSkillsModel.NameRole:
            result = row.name
        elif role == QSkillsModel.IdRole:
            result = skill_id
        elif role == QSkillsModel.WordsRole:
            result = row.words
        elif role == QSkillsModel.PhrasesRole:
            result = row.phrases
        elif role == QSkillsModel.ImageSetRole:
            result = row.image_set
        elif role == QSkillsModel.DictionaryRole:
            result = row.dictionary
        elif role == QSkillsModel.IntroductionRole:
            result = row.introduction
        elif role == QSkillsModel.SampleWordsStrRole:
            if len(row.words) > 3:
                result = "{0}, {1}, {2}".format(
                    row.words[0].in_target_language[0],
                    row.words[1].in_target_language[0],
                    row.words[2].in_target_language[0])
            else:
                result = ""
        elif role == QSkillsModel.SamplePhrasesStrRole:
            if len(row.phrases) > 3:
                result = "{0}, {1}, {2}".format(
                    row.phrases[0].in_target_language[0],
                    row.phrases[1].in_target_language[0],
                    row.phrases[2].in_target_language[0])
            else:
                result = ""
        elif role == QSkillsModel.SampleImageRole:
            if row.image_set:
                result = row.image_set[0]
            else:
                result = ""
        elif role == QSkillsModel.ActiveUserProgress:
            if stats and stats["passed"] and stats["total_skill_challenges"]:
                result = len(stats["passed"])/stats["total_skill_challenges"]
            else:
                result = 0
        elif role == QSkillsModel.ActiveUserSkillStats:
            result = stats

        return result

    def readSkills(self):
        """Returns the list that represents the skills of a course"""
        return self._skills

    def setSkills(self, val):
        """Sets the list that represents the skills of a course"""
        self.beginResetModel()
        self._skills = val
        self.skillsChanged.emit()
        self.endResetModel()

    @Signal
    def skillsChanged(self):
        """Signal that is emitted when the skills list property is set or changed"""

    @Slot()
    def updateModel(self):
        """Slot that is called when the model is reset"""
        self.beginResetModel()
        self.endResetModel()

    skills = Property(list, readSkills, setSkills, notify=skillsChanged)
