# SPDX-FileCopyrightText: 2021 Dimitris Kardarakos <dimkard@posteo.net>
# SPDX-License-Identifier: GPL-3.0-or-later

from dataclasses import asdict
import json
from random import shuffle, choice, sample
from slugify import slugify
from librelingo_utils import clean_word
from glossaico.q_data_controller import QDataController
from glossaico.abstract_practice_strategy import AbstractPracticeStrategy
from glossaico.challenge_definition import (
    ShortInputChallengeQuestion, ChallengeAnswer,
    OptionsChallengeQuestion, CardsChallengeQuestion,
    ListeningChallengeQuestion, ChipsChallengeQuestion
    )
from glossaico.q_config_singleton_adapter import QConfigSingletonAdapter


class SimplePracticeStrategy(AbstractPracticeStrategy):
    """AbstractPracticeStrategy class that:
    - Within each challenge session asks again challenges answered incorrectly
    - Removes from the repeat queue a challenge if answered correctly
    at first asking within a challenge session
    """

    challenges_per_practice = 10
    option_challenges_amt = 3
    cardsChallenges_amt = 4

    def __init__(self):
        self._cfg = QConfigSingletonAdapter.getInstance()
        self._skill_id = ""
        self._skill_name = ""
        self._skill_words = []
        self._skill_phrases = []
        self._total_skill_challenges_amt = 0
        self._session_challenges_amt = 0
        self._session_challenges = []
        self._session_challenges_need_repeat = []
        self._total_challenges_passed = []
        self._session_correct = 0
        self._session_incorrect = 0
        self._current_challenge = {}
        self._current_question = None
        self._current_correct_answer = None
        self._repeat_mode = False
        self._revise_mode = False
        self._q_data_controller = QDataController.getInstance()

        self._function_map = {
            "shortInput": self._shortInputChallenge,
            "options": self._optionsChallenge,
            "cards": self._cardsChallenge,
            "listeningExercise": self._listeningChallenge,
            "chips": self._chipsChallenge
        }

    @property
    def currentChallengeQuestion(self):
        return asdict(self._current_question)

    @property
    def practiceProgress(self):
        return (self._session_challenges_amt
                - len(self._session_challenges) - 1) / self._session_challenges_amt

    @property
    def challengesAnswered(self):
        return self._session_correct + self._session_incorrect

    def startPractice(self, course_name, skill_id, skill_name, skill_words,
                      skill_phrases, user_stats):
        if course_name and skill_name:
            self._skill_name = skill_name
            self._skill_words = skill_words
            self._skill_phrases = skill_phrases
            self._skill_id = skill_id
            self._session_incorrect = 0
            self._session_correct = 0
            self._session_challenges_need_repeat = []

            slug_skill_name = slugify(self._skill_name)
            app_folder = self._cfg.applicationDirectory

            with open("{}/courses/export/{}/challenges/{}.json".
                      format(app_folder, course_name, slug_skill_name
                             ), "r", encoding="utf-8") as challenge_file:
                challenge_file_data = json.load(challenge_file)
                skill_challenges = sorted(
                    challenge_file_data['challenges'],
                    key=lambda x: (x['group'], x['priority']))
                valid_skill_challenges = self._validChallenges(
                    skill_challenges, course_name)
                self._total_skill_challenges_amt = len(valid_skill_challenges)
                self._total_challenges_passed = user_stats["passed"]
                skill_challenges_for_practice = [x for x in valid_skill_challenges if (
                    x["id"] not in self._total_challenges_passed)]

                # user is in revise mode, just serve a set of random challenges
                if len(skill_challenges_for_practice) == 0:
                    self._session_challenges = sample(
                        valid_skill_challenges, SimplePracticeStrategy.challenges_per_practice)
                    self._revise_mode = True
                elif (len(skill_challenges_for_practice)
                      >= SimplePracticeStrategy.challenges_per_practice):
                    self._session_challenges = skill_challenges_for_practice[
                        0:SimplePracticeStrategy.challenges_per_practice]
                else:
                    self._session_challenges = skill_challenges_for_practice

                if self._cfg.silentMode:
                    self._session_challenges = [
                        x for x in self._session_challenges if x["type"] != "listeningExercise"]

                self._session_challenges.reverse()
                self._session_challenges_amt = len(self._session_challenges)
                #print("We are going to practice {}".format(self._session_challenges))

                return True

        return False

    def loadChallenge(self):
        if (not self._session_challenges) and (not self._session_challenges_need_repeat):
            return False

        if self._session_challenges:
            self._current_challenge = self._session_challenges.pop()
            self._repeat_mode = False
        elif self._session_challenges_need_repeat:
            self._current_challenge = self._session_challenges_need_repeat.pop()
            self._repeat_mode = True

        data_func = self._function_map[self._current_challenge["type"]]
        data_func()

        return True

    def checkResult(self, answer_given):
        is_correct = False

        if self._current_question.challengeType == "shortInput":
            is_correct = answer_given["answer"].lower() in [f.lower(
            ) for f in self._current_correct_answer.formInTargetLanguage]
        elif self._current_question.challengeType == "chips":
            correct_forms = [
                f for f in self._current_correct_answer.formInTargetLanguage]
            is_correct = (answer_given["answer"] in correct_forms)
        else:
            is_correct = clean_word(answer_given["answer"].lower()) == clean_word(
                self._current_correct_answer.formInTargetLanguage.lower())

        if not is_correct:
            self._session_challenges_need_repeat.append(
                self._current_challenge)

        if not self._repeat_mode:
            self._addToStats(is_correct)

        return is_correct

    def _addToStats(self, result):
        if result:
            self._session_correct = self._session_correct + 1
            self._total_challenges_passed.append(self._current_challenge["id"])
        else:
            self._session_incorrect = self._session_incorrect + 1

    def updateStats(self):
        if not self._revise_mode:
            self._cfg.updateActiveUserSkillStats(
                self._skill_id, self._total_challenges_passed,
                self._total_skill_challenges_amt)

    def _chipsChallenge(self):
        challenge_chips = self._current_challenge["chips"]
        shuffle(challenge_chips)
        self._current_question = ChipsChallengeQuestion(
            cid=self._current_challenge["id"],
            group=self._current_challenge["group"],
            priority=self._current_challenge["priority"],
            meaningInSourceLanguage=' '.join(
                str(x["word"]) for x in self._current_challenge["phrase"]),
            chips=challenge_chips
        )

        self._current_correct_answer = ChallengeAnswer(
            cid=self._current_challenge["id"],
            formInTargetLanguage=self._current_challenge["solutions"]
        )

    def _listeningChallenge(self):
        self._current_question = ListeningChallengeQuestion(
            cid=self._current_challenge["id"],
            group=self._current_challenge["group"],
            priority=self._current_challenge["priority"],
            meaning=self._current_challenge["meaning"],
            audio=self._current_challenge["audio"])

        self._current_correct_answer = ChallengeAnswer(
            cid=self._current_challenge["id"],
            formInTargetLanguage=self._current_challenge["answer"]
        )

    def _shortInputChallenge(self):
        self._current_question = ShortInputChallengeQuestion(
            cid=self._current_challenge["id"],
            group=self._current_challenge["group"],
            priority=self._current_challenge["priority"],
            meaningInSourceLanguage=' '.join(
                str(x["word"]) for x in self._current_challenge["phrase"]),
            pictures=self._current_challenge["pictures"])

        self._current_correct_answer = ChallengeAnswer(
            cid=self._current_challenge["id"],
            formInTargetLanguage=self._current_challenge["formInTargetLanguage"]
        )

    def _optionsChallenge(self):
        challenge_options = [self._current_challenge["formInTargetLanguage"]]

        candidates = [p.in_target_language[0] for p in self._skill_phrases]
        candidates.remove(self._current_challenge["formInTargetLanguage"])

        for i in range(SimplePracticeStrategy.option_challenges_amt - 1):
            challenge_options.append(candidates[i])

        shuffle(challenge_options)

        self._current_question = OptionsChallengeQuestion(
            cid=self._current_challenge["id"],
            group=self._current_challenge["group"],
            priority=self._current_challenge["priority"],
            meaningInSourceLanguage=self._current_challenge["meaningInSourceLanguage"],
            options=challenge_options)

        self._current_correct_answer = ChallengeAnswer(
            cid=self._current_challenge["id"],
            formInTargetLanguage=self._current_challenge["formInTargetLanguage"]
        )

    def _cardsChallenge(self):
        card_options = [{"formInTargetLanguage": self._current_challenge["formInTargetLanguage"],
                         "picture":  choice(self._current_challenge["pictures"]).split('.')[0]}]

        # create a list of candidate cards
        candidates = [p for p in self._skill_words]

        # populate the list of card options
        for c in candidates:
            # avoid adding again the correct answer
            if ((self._current_challenge["formInTargetLanguage"]
                 not in c.in_target_language) and c.pictures):
                new_card = {"formInTargetLanguage": c.in_target_language[0],
                            "picture": choice(c.pictures).split('.')[0]}
                card_options.append(new_card)

            if len(card_options) == SimplePracticeStrategy.cardsChallenges_amt:
                break

        shuffle(card_options)

        self._current_question = CardsChallengeQuestion(
            cid=self._current_challenge["id"],
            group=self._current_challenge["group"],
            priority=self._current_challenge["priority"],
            meaningInSourceLanguage=self._current_challenge["meaningInSourceLanguage"],
            options=card_options
        )

        self._current_correct_answer = ChallengeAnswer(
            cid=self._current_challenge["id"],
            formInTargetLanguage=self._current_challenge["formInTargetLanguage"]
        )

    def _validChallenges(self, challenges, course_name):
        challenges = [c for c in challenges if (c["type"] != "listeningExercise") or (
            (c["type"] == "listeningExercise")
            and (self._q_data_controller.audioExists(course_name, c["audio"])))]

        challenges = [c for c in challenges if (c["type"] != "cards") or (
            (c["type"] == "cards") and c["pictures"]
            and len([p for p in c["pictures"]
                     if not self._q_data_controller.imageExists(p, "tiny")]) == 0)]
        return challenges
