# SPDX-FileCopyrightText: 2021 Dimitris Kardarakos <dimkard@posteo.net>
# SPDX-License-Identifier: GPL-3.0-or-later

import os
from pathlib import Path
from yaml import load, dump, Loader, YAMLError
from PySide2.QtCore import QStandardPaths
from glossaico import APP_COMPONENT_NAME
from glossaico.abstract_configurator import AbstractConfigurator


class YamlConfigurator(AbstractConfigurator):
    """AbstractConfigurator class that makes uses of yaml file for
    configuration storage
    """

    def __init__(self):

        self._cfg_data = None

        config_home = QStandardPaths.writableLocation(QStandardPaths.GenericConfigLocation)
        if not config_home:
            raise NotADirectoryError("XDG configuration folder not found")

        config_folder = Path(config_home, APP_COMPONENT_NAME)
        if not config_folder.is_dir():
            config_folder.mkdir()

        self._config_file_path = config_folder / 'general.yaml'
        if not self._config_file_path.is_file():
            with open(self._config_file_path, 'w'):
                pass

        self._loadConfig()

    @staticmethod
    def dataDirectory():
        data_home = QStandardPaths.writableLocation(QStandardPaths.GenericDataLocation)
        if not data_home:
            raise NotADirectoryError("Data folder not found")

        app_data_folder = Path(data_home, APP_COMPONENT_NAME)

        if not app_data_folder.is_dir():
            app_data_folder.mkdir()

        return "{}/{}".format(data_home, APP_COMPONENT_NAME)

    @staticmethod
    def cacheDirectory():
        cache_home = QStandardPaths.writableLocation(QStandardPaths.GenericCacheLocation)
        if not cache_home:
            raise NotADirectoryError("Cache folder not found")

        cache_data_folder = Path(cache_home, APP_COMPONENT_NAME)

        if not cache_data_folder.is_dir():
            cache_data_folder.mkdir()

        return "{}/{}".format(cache_home, APP_COMPONENT_NAME)

    @staticmethod
    def configDirectory():
        return QStandardPaths.writableLocation(
            QStandardPaths.GenericConfigLocation) / APP_COMPONENT_NAME

    @staticmethod
    def applicationDirectory():
        return os.path.abspath(os.path.dirname(__file__))

    def userSkillStats(self, uid, course_name, skill_id):

        skill = self._skill(uid, course_name, skill_id)

        if "stats" not in skill:
            self._cfg_data["users"][uid]["courses"][course_name]["skills"][skill_id]["stats"] = {
                "passed": [], "total_skill_challenges": 0}
            self._flushConfig()

        return self._cfg_data["users"][uid]["courses"][course_name]["skills"][skill_id]["stats"]

    def updateUserSkillStats(self, uid, course_name, skill_id,
                             challenges_passed, total_skill_challenges):
        self._cfg_data["users"][uid]["courses"][course_name]["skills"][skill_id]["stats"] = {
            "passed": challenges_passed,
            "total_skill_challenges": total_skill_challenges
            }
        self._flushConfig()

    def activeUserSkillStats(self, skill_id):
        return self.userSkillStats(self.activeUserId,
                                   self.activeCourseName, skill_id)

    def updateActiveUserSkillStats(self, skill_id,
                                   challenges_passed, total_skill_challenges):
        self.updateUserSkillStats(self.activeUserId, self.activeCourseName,
                                  skill_id, challenges_passed, total_skill_challenges)

    @property
    def configVersion(self):
        cfg_ver = ""
        if "config-version" in self._cfg_data:
            cfg_ver = self._cfg_data["config-version"]
        else:
            self._cfg_data["config-version"] = cfg_ver
            self._flushConfig()

        return cfg_ver

    @configVersion.setter
    def configVersion(self, val):
        self._cfg_data["config-version"] = val
        self._flushConfig()

    @property
    def activeUserId(self):
        auid = "default-user"
        if "active-user-id" in self._cfg_data:
            auid = self._cfg_data["active-user-id"]
        else:
            self._cfg_data["active-user-id"] = auid
            self._flushConfig()

        return auid

    @activeUserId.setter
    def activeUserId(self, val):
        self._cfg_data["active-user-id"] = val
        self._flushConfig()

    @property
    def activeCourseName(self):
        acn = ""
        if "active-course-name" in self._cfg_data:
            acn = self._cfg_data["active-course-name"]
        else:
            self._cfg_data["active-course-name"] = acn
            self._flushConfig()

        return acn

    @activeCourseName.setter
    def activeCourseName(self, val):
        self._cfg_data["active-course-name"] = val
        self._flushConfig()

    @property
    def silentMode(self):
        silent_mode = False
        if "silent-mode" in self._cfg_data:
            silent_mode = self._cfg_data["silent-mode"]
        else:
            self._cfg_data["silent-mode"] = silent_mode
            self._flushConfig()

        return silent_mode

    @silentMode.setter
    def silentMode(self, val):
        self._cfg_data["silent-mode"] = val
        self._flushConfig()

    @property
    def imagesReady(self):
        images_ready = False
        if "images-ready" in self._cfg_data:
            images_ready = self._cfg_data["images-ready"]
        else:
            self._cfg_data["images-ready"] = images_ready
            self._flushConfig()

        return images_ready

    @imagesReady.setter
    def imagesReady(self, val):
        self._cfg_data["images-ready"] = val
        self._flushConfig()

    def _loadConfig(self):
        with open(self._config_file_path, "r") as cfg:
            try:
                self._cfg_data = load(cfg, Loader=Loader)
                if not self._cfg_data:
                    self._cfg_data = {}
                    self._flushConfig()
            except YAMLError as exc:
                print(exc)
        if self.configVersion != "v1":
            self._migrateConfiguration("v1")

    def _flushConfig(self):
        with open(self._config_file_path, 'w') as yaml_file:
            yaml_file.write(dump(self._cfg_data, default_flow_style=False))

    def _migrateConfiguration(self, target_version):
        if target_version == "v1":
            self._createUsers()
            self.activeUserId = 'default-user'
            self.configVersion = "v1"

    def _createUserCourses(self, uid):
        self._cfg_data["users"][uid]["courses"] = {}
        self._flushConfig()

    def _course(self, uid, course_name):
        user = self._user(uid)

        if "courses" not in user:
            self._createUserCourses(uid)

        if course_name not in self._cfg_data["users"][uid]["courses"]:
            self._cfg_data["users"][uid]["courses"][course_name] = {}
            self._flushConfig()

        return self._cfg_data["users"][uid]["courses"][course_name]

    def _user(self, uid):
        if ("users" not in self._cfg_data) or (uid not in self._cfg_data["users"]):
            self._addUser(uid)

        return self._cfg_data["users"][uid]

    def _createUsers(self):
        self._cfg_data["users"] = {}
        self._flushConfig()

    def _addUser(self, uid):
        if "users" not in self._cfg_data:
            self._createUsers()

        if uid not in self._cfg_data["users"]:
            self._cfg_data["users"][uid] = {}
            self._flushConfig()

    def _skill(self, uid, course_name, skill_id):
        user_course = self._course(uid, course_name)

        if "skills" not in user_course:
            self._createSkills(uid, course_name)

        if skill_id not in self._cfg_data["users"][uid]["courses"][course_name]["skills"]:
            self._cfg_data["users"][uid]["courses"][course_name]["skills"][skill_id] = {}
            self._flushConfig()

        return self._cfg_data["users"][uid]["courses"][course_name]["skills"][skill_id]

    def _createSkills(self, uid, course_name):
        self._cfg_data["users"][uid]["courses"][course_name]["skills"] = {
        }
        self._flushConfig()
